@@ -16,6 +16,19 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+/*
+ * $Log: dao.h,v $
+ * Revision 1.1.1.1  2000/02/05 01:35:20  llanero
+ * Uploaded cdrdao 1.1.3 with pre10 patch applied.
+ *
+ * Revision 1.3  1999/01/24 16:03:57  mueller
+ * Applied Radek Doulik's ring buffer patch. Added some cleanups and
+ * improved behavior in error case.
+ *
+ * Revision 1.2  1998/08/15 20:47:16  mueller
+ * Added support for GenericMMC driver.
+ *
+ */
 
 #ifndef __DAO_H__
 #define __DAO_H__
@@ -23,6 +36,7 @@
 #include "Toc.h"
 #include "CdrDriver.h"
 
-int writeDiscAtOnce(const Toc *, CdrDriver *, int, int);
+int writeDiskAtOnce(const Toc *, CdrDriver *, int nofBuffers, int swap,
+		    int testMode, int remoteMode);
 
 #endif
