@@ -18,6 +18,36 @@
  */
 /*
  * $Log: SampleDisplay.cc,v $
+ * Revision 1.7  2000/04/23 09:07:08  andreasm
+ * * Fixed most problems marked with '//llanero'.
+ * * Added audio CD edit menus to MDIWindow.
+ * * Moved central storage of TocEdit object to MDIWindow.
+ * * AudioCdChild is now handled like an ordinary non modal dialog, i.e.
+ *   it has a normal 'update' member function now.
+ * * Added CdTextTable modal dialog.
+ * * Old functionality of xcdrdao is now available again.
+ *
+ * Revision 1.6  2000/04/17 21:30:49  andreasm
+ * Fixed sample display.
+ * AudioCDChild displays animated cursor again when playing.
+ *
+ * Revision 1.5  2000/04/14 13:22:02  llanero
+ * changed the MDI object to GnomeApp until gnome-- MDI is done.
+ * Also catched a bug in SampleDisplay.cc:1000.
+ *
+ * Revision 1.4  2000/03/05 22:25:52  llanero
+ * more code translated to gtk-- 1.1.8
+ *
+ * Revision 1.3  2000/03/04 01:28:52  llanero
+ * SampleDisplay.{cc,h} are fixed now = gtk 1.1.8 compliant.
+ *
+ * Revision 1.2  2000/02/20 23:34:54  llanero
+ * fixed scsilib directory (files mising ?-()
+ * ported xdao to 1.1.8 / gnome (MDI) app
+ *
+ * Revision 1.1.1.1  2000/02/05 01:39:50  llanero
+ * Uploaded cdrdao 1.1.3 with pre10 patch applied.
+ *
  * Revision 1.5  1999/08/07 16:27:28  mueller
  * Applied patch from Yves Bastide:
  * * prefixing member function names with their class name in connect_to_method
@@ -34,7 +64,7 @@
  *
  */
 
-static char rcsid[] = "$Id$";
+static char rcsid[] = "$Id: SampleDisplay.cc,v 1.7 2000/04/23 09:07:08 andreasm Exp $";
 
 #include <stdio.h>
 #include <limits.h>
@@ -131,9 +161,8 @@ static gchar *INDEX_EXTEND_XPM_DATA[] = {
 
 SampleDisplay::SampleDisplay()
 {
-  adjustment_ = new Gtk_Adjustment(0.0, 0.0, 1.0);
-  connect_to_method(adjustment_->value_changed, this,
-		    &SampleDisplay::scrollTo);
+  adjustment_ = new Gtk::Adjustment(0.0, 0.0, 1.0);
+  adjustment_->value_changed.connect(slot(this, &SampleDisplay::scrollTo));
 
   trackManager_ = NULL;
 
@@ -168,23 +197,15 @@ SampleDisplay::SampleDisplay()
   selectedTrack_ = 0;
   selectedIndex_ = 0;
 
-  connect_to_method(expose_event, this,
-                    &SampleDisplay::handle_expose_event);
-  connect_to_method(configure_event, this,
-                    &SampleDisplay::handle_configure_event);
-  connect_to_method(motion_notify_event, this,
-                    &SampleDisplay::handle_motion_notify_event);
-  connect_to_method(button_press_event, this,
-                    &SampleDisplay::handleButtonPressEvent);
-  connect_to_method(button_release_event, this,
-                    &SampleDisplay::handleButtonReleaseEvent);
-  connect_to_method(enter_notify_event, this,
-                    &SampleDisplay::handleEnterEvent);
-  connect_to_method(leave_notify_event, this,
-                    &SampleDisplay::handleLeaveEvent);
-  
-  //connect_to_method (motion_notify_event, this, &handle_motion_notify_event);
-  //connect_to_method (button_press_event, this, &handle_button_press_event);
+  expose_event.connect(slot(this, &SampleDisplay::handle_expose_event));
+  configure_event.connect(slot(this, &SampleDisplay::handle_configure_event));
+  motion_notify_event.connect(slot(this,
+  		&SampleDisplay::handle_motion_notify_event));
+  button_press_event.connect(slot(this, &SampleDisplay::handleButtonPressEvent));
+  button_release_event.connect(slot(this,
+  		&SampleDisplay::handleButtonReleaseEvent));
+  enter_notify_event.connect(slot(this, &SampleDisplay::handleEnterEvent));
+  leave_notify_event.connect(slot(this, &SampleDisplay::handleLeaveEvent));
 
   set_events (GDK_EXPOSURE_MASK
 	      | GDK_LEAVE_NOTIFY_MASK
@@ -215,10 +236,18 @@ void SampleDisplay::setTocEdit(TocEdit *t)
   }
 }
 
-void SampleDisplay::updateToc()
+void SampleDisplay::updateToc(unsigned long smin, unsigned long smax)
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
+  if (smin <= smax) {
+    minSample_ = smin;
+    maxSample_ = smax;
+  }
+
   if (toc->length().samples() == 0) {
     minSample_ = maxSample_ = 0;
   }
@@ -242,6 +271,9 @@ void SampleDisplay::updateToc()
 
 void SampleDisplay::setView(unsigned long start, unsigned long end)
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
   if (end < start)
@@ -331,6 +363,9 @@ void SampleDisplay::setSelectedTrackMarker(int trackNr, int indexNr)
 
 void SampleDisplay::setRegion(unsigned long start, unsigned long end)
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
   if (end <= start || end >= toc->length().samples()) {
@@ -383,6 +418,9 @@ void SampleDisplay::getColor(const char *colorName, Gdk_Color *color)
 
 void SampleDisplay::scrollTo()
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
   GtkAdjustment *adjust = adjustment_->gtkobj();
 
@@ -406,6 +444,9 @@ void SampleDisplay::scrollTo()
 
 unsigned long SampleDisplay::pixel2sample(gint x)
 {
+  if (tocEdit_ == NULL)
+    return 0;
+
   Toc *toc = tocEdit_->toc();
   unsigned long sample;
 
@@ -458,47 +499,52 @@ gint SampleDisplay::sample2pixel(unsigned long sample)
 
 int SampleDisplay::handle_configure_event (GdkEventConfigure *event)
 {
-  if (drawGc_ == NULL) {
+  if (!drawGc_) {
     Gdk_Bitmap mask;
     Gdk_Window window(get_window());
 
-    drawGc_ = new Gdk_GC(window);
+    drawGc_ = Gdk_GC(window);
+        
     getColor("darkslateblue", &sampleColor_);
     getColor("red3", &middleLineColor_);
     getColor("gold2", &cursorColor_);
     getColor("red", &markerColor_);
     getColor("#ffc0e0", &selectionBackgroundColor_);
 
-    timeTickFont_ = new Gdk_Font("fixed");
-    drawGc_->set_font(*timeTickFont_);
+    timeTickFont_ = Gdk_Font("fixed");
+    drawGc_.set_font(timeTickFont_);
 
     trackMarkerFont_ = timeTickFont_;
 
-    timeLineHeight_ = timeTickFont_->char_height('0') + 6;
-    trackLineHeight_ = timeTickFont_->char_height('0') + 10;
+    timeLineHeight_ = timeTickFont_.char_height('0') + 6;
+    trackLineHeight_ = timeTickFont_.char_height('0') + 10;
 
-    trackMarkerPixmap_ = new Gdk_Pixmap;
-    trackMarkerPixmap_->create_from_xpm_d(window, mask,
+    trackMarkerPixmap_ = Gdk_Pixmap();
+    trackMarkerPixmap_.create_from_xpm_d(window, mask,
 					  get_colormap().white(),
 					  TRACK_MARKER_XPM_DATA);
-    indexMarkerPixmap_ = new Gdk_Pixmap;
-    indexMarkerPixmap_->create_from_xpm_d(window, mask,
+
+    indexMarkerPixmap_ = Gdk_Pixmap();
+    indexMarkerPixmap_.create_from_xpm_d(window, mask,
 					  get_colormap().white(),
 					  INDEX_MARKER_XPM_DATA);
-    trackMarkerSelectedPixmap_ = new Gdk_Pixmap;
-    trackMarkerSelectedPixmap_->create_from_xpm_d(window, mask,
+
+    trackMarkerSelectedPixmap_ = Gdk_Pixmap();
+    trackMarkerSelectedPixmap_.create_from_xpm_d(window, mask,
 						  markerColor_,
 						  TRACK_MARKER_XPM_DATA);
-    indexMarkerSelectedPixmap_ = new Gdk_Pixmap;
-    indexMarkerSelectedPixmap_->create_from_xpm_d(window, mask,
+						  
+    indexMarkerSelectedPixmap_ = Gdk_Pixmap();
+    indexMarkerSelectedPixmap_.create_from_xpm_d(window, mask,
 						  markerColor_,
 						  INDEX_MARKER_XPM_DATA);
-    trackExtendPixmap_ = new Gdk_Pixmap;
-    trackExtendPixmap_->create_from_xpm_d(window, mask,
+    trackExtendPixmap_ = Gdk_Pixmap();
+    trackExtendPixmap_.create_from_xpm_d(window, mask,
 					  get_colormap().white(),
 					  TRACK_EXTEND_XPM_DATA);
-    indexExtendPixmap_ = new Gdk_Pixmap;
-    indexExtendPixmap_->create_from_xpm_d(window, mask,
+
+    indexExtendPixmap_ = Gdk_Pixmap();
+    indexExtendPixmap_.create_from_xpm_d(window, mask,
 					  get_colormap().white(),
 					  INDEX_EXTEND_XPM_DATA);
 
@@ -508,6 +554,7 @@ int SampleDisplay::handle_configure_event (GdkEventConfigure *event)
 
   width_ = width();
   height_ = height();
+
   chanHeight_ = (height_ - timeLineHeight_ - trackLineHeight_ - 2) / 2;
 
   lcenter_ = chanHeight_ / 2 + trackLineHeight_;
@@ -522,14 +569,15 @@ int SampleDisplay::handle_configure_event (GdkEventConfigure *event)
   sampleEndX_ = width_ - 10;
   sampleWidthX_ = sampleEndX_ - sampleStartX_ + 1;
   
-  if (pixmap_ != NULL)
-    delete pixmap_;
-
-  pixmap_ = new Gdk_Pixmap(get_window(), width(), height(), -1);
+  if (pixmap_)
+    pixmap_.release();
+      
+  pixmap_ = Gdk_Pixmap(get_window(), width(), height(), -1);
 
   //message(0, "handle_configure_event: %d\n", width_);
 
-  updateSamples();
+  if (width_ > 100 && height_ > 100)
+    updateSamples();
 
   return TRUE;
 }
@@ -691,10 +739,10 @@ int SampleDisplay::handle_motion_notify_event (GdkEventMotion *event)
     }
 
     if (dw != 0) {
-      drawGc_->set_foreground(cursorColor_);
-      drawGc_->set_function(GDK_XOR);
-      get_window().draw_rectangle(*drawGc_, TRUE, dx, 0, dw, height_ - 1);
-      drawGc_->set_function(GDK_COPY);
+      drawGc_.set_foreground(cursorColor_);
+      drawGc_.set_function(GDK_XOR);
+      get_window().draw_rectangle(drawGc_, TRUE, dx, 0, dw, height_ - 1);
+      drawGc_.set_function(GDK_COPY);
     }      
 
     selectionEnd_ = x;
@@ -744,7 +792,7 @@ int SampleDisplay::handleLeaveEvent(GdkEventCrossing *event)
 void SampleDisplay::redraw(gint x, gint y, gint width, gint height,
 			   int drawMask)
 {
-  get_window().draw_pixmap(*drawGc_, *pixmap_, x, y, x, y, width, height);
+  get_window().draw_pixmap(drawGc_, pixmap_, x, y, x, y, width, height);
 
   if ((drawMask & 0x02) == 0)
     drawMarker();
@@ -758,11 +806,11 @@ void SampleDisplay::redraw(gint x, gint y, gint width, gint height,
 void SampleDisplay::drawMarker()
 {
   if (markerSet_) {
-    drawGc_->set_foreground(markerColor_);
+    drawGc_.set_foreground(markerColor_);
 
     markerX_ = sample2pixel(markerSample_);
     if (markerX_ >= 0) 
-      get_window().draw_line(*drawGc_, markerX_, trackLineY_,
+      get_window().draw_line(drawGc_, markerX_, trackLineY_,
 			     markerX_, height_ - 1);
   }
 }
@@ -788,15 +836,18 @@ void SampleDisplay::clearMarker()
 
 void SampleDisplay::updateSamples()
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
-  if (pixmap_ == NULL)
+  if (!pixmap_)
     return;
 
   gint halfHeight = chanHeight_ / 2;
 
-  drawGc_->set_foreground(get_colormap().white());
-  pixmap_->draw_rectangle(*drawGc_, TRUE, 0, 0, width_, height_);
+  drawGc_.set_foreground(get_colormap().white());
+  pixmap_.draw_rectangle(drawGc_, TRUE, 0, 0, width_, height_);
   
   long res = (maxSample_ - minSample_ + 1)/sampleWidthX_;
   long bres = res / tocEdit_->sampleManager()->blocking();
@@ -825,20 +876,21 @@ void SampleDisplay::updateSamples()
     }
 
     if (regionStart >= 0 && regionEnd >= regionStart) {
-      drawGc_->set_foreground(selectionBackgroundColor_);
-      pixmap_->draw_rectangle(*drawGc_, TRUE,
+      drawGc_.set_foreground(selectionBackgroundColor_);
+      pixmap_.draw_rectangle(drawGc_, TRUE,
 			      regionStart, lcenter_ - halfHeight,
 			      regionEnd - regionStart + 1, chanHeight_);
-      pixmap_->draw_rectangle(*drawGc_, TRUE,
+      pixmap_.draw_rectangle(drawGc_, TRUE,
 			      regionStart, rcenter_ - halfHeight,
 			      regionEnd - regionStart + 1, chanHeight_);
     }
   }
 
-  drawGc_->set_foreground(sampleColor_);
+  drawGc_.set_foreground(sampleColor_);
 
   if (bres > 0) {
     //message(0, "Draw 1");
+
     for (s = minSample_, i = sampleStartX_;
 	 s < maxSample_ && i <= sampleEndX_;
 	 s += res, i++) {
@@ -846,11 +898,11 @@ void SampleDisplay::updateSamples()
 
       if (regionStart != -1 && i >= regionStart && regionActive == 0) {
 	regionActive = 1;
-	drawGc_->set_foreground(markerColor_);
+	drawGc_.set_foreground(markerColor_);
       }
       else if (regionActive == 1 && i > regionEnd) {
 	regionActive = 2;
-	drawGc_->set_foreground(sampleColor_);
+	drawGc_.set_foreground(sampleColor_);
       }
 
       tocEdit_->sampleManager()->getPeak(s, s + res, &lnegsum, &lpossum,
@@ -860,28 +912,29 @@ void SampleDisplay::updateSamples()
       pos /= SHRT_MAX;
       //pos /= bres;
       if (pos != 0)
-	pixmap_->draw_line(*drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
+	pixmap_.draw_line(drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
       
       pos = double(lpossum) * halfHeight;
       pos /= SHRT_MAX;
       //pos /= bres;
       if (pos != 0)
-	pixmap_->draw_line(*drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
+	pixmap_.draw_line(drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
       
       pos = double(rnegsum) * halfHeight;
       pos /= SHRT_MAX;
       //pos /= bres;
       if (pos != 0)
-	pixmap_->draw_line(*drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
+	pixmap_.draw_line(drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
       
       pos = double(rpossum) * halfHeight;
       pos /= SHRT_MAX;
       //pos /= bres;
       if (pos != 0)
-	pixmap_->draw_line(*drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
+	pixmap_.draw_line(drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
     }
   }
   else if (maxSample_ > 0 && res >= 1) {
+
     //message(0, "Draw 2: %ld < %ld", res,
     //        tocEdit_->sampleManager()->blocking());
     TocReader reader(toc);
@@ -916,36 +969,36 @@ void SampleDisplay::updateSamples()
 
 	if (regionStart != -1 && i >= regionStart && regionActive == 0) {
 	  regionActive = 1;
-	  drawGc_->set_foreground(markerColor_);
+	  drawGc_.set_foreground(markerColor_);
 	}
 	else if (regionActive == 1 && i > regionEnd) {
 	  regionActive = 2;
-	  drawGc_->set_foreground(sampleColor_);
+	  drawGc_.set_foreground(sampleColor_);
 	}
 
 	pos = double(lnegsum) * halfHeight;
 	pos /= SHRT_MAX;
 	//pos /= bres;
 	if (pos != 0)
-	  pixmap_->draw_line(*drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
+	  pixmap_.draw_line(drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
       
 	pos = double(lpossum) * halfHeight;
 	pos /= SHRT_MAX;
 	//pos /= bres;
 	if (pos != 0)
-	  pixmap_->draw_line(*drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
+	  pixmap_.draw_line(drawGc_, i, lcenter_, i, lcenter_ - (gint)pos);
 	
 	pos = double(rnegsum) * halfHeight;
 	pos /= SHRT_MAX;
 	//pos /= bres;
 	if (pos != 0)
-	  pixmap_->draw_line(*drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
+	  pixmap_.draw_line(drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
 	
 	pos = double(rpossum) * halfHeight;
 	pos /= SHRT_MAX;
 	//pos /= bres;
 	if (pos != 0)
-	  pixmap_->draw_line(*drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
+	  pixmap_.draw_line(drawGc_, i, rcenter_, i, rcenter_ - (gint)pos);
       }
 
       delete[] sampleBuf;
@@ -960,11 +1013,13 @@ void SampleDisplay::updateSamples()
     if (reader.openData() == 0) {
       long len = maxSample_ - minSample_ + 1;
       Sample *sampleBuf = new Sample[len];
+
       double pres = double(sampleWidthX_ - 1) / double(len - 1);
       double di;
       gint pos1;
       gint lastPosLeft, lastPosRight;
 
+
       //printf("Drawing exact, res=%ld pres=%g %ld-%ld\n", res, pres, minSample_, maxSample_);
 
       if (reader.seekSample(minSample_) == 0 &&
@@ -977,11 +1032,11 @@ void SampleDisplay::updateSamples()
 	      minSample_ + j - 1 >= regionStartSample_ &&
 	      minSample_ + j <= regionEndSample_) {
 	    regionActive = 1;
-	    drawGc_->set_foreground(markerColor_);
+	    drawGc_.set_foreground(markerColor_);
 	  }
 	  else if (regionActive == 1 && minSample_ + j > regionEndSample_) {
 	    regionActive = 2;
-	    drawGc_->set_foreground(sampleColor_);
+	    drawGc_.set_foreground(sampleColor_);
 	  }
 
 	  pos = sampleBuf[j - 1].left() * halfHeight;
@@ -992,7 +1047,7 @@ void SampleDisplay::updateSamples()
 	  lastPosLeft = pos1;
 
 	  if (pos != 0 || pos1 != 0)
-	    pixmap_->draw_line(*drawGc_, long(di - pres), lcenter_ - (gint)pos,
+	    pixmap_.draw_line(drawGc_, long(di - pres), lcenter_ - (gint)pos,
 			       long(di), lcenter_ - pos1);
 
 	  pos = sampleBuf[j - 1].right() * halfHeight;
@@ -1003,7 +1058,7 @@ void SampleDisplay::updateSamples()
 	  lastPosRight = pos1;
 
 	  if (pos != 0 || pos1 != 0)
-	    pixmap_->draw_line(*drawGc_, long(di - pres), rcenter_ - (gint)pos,
+	    pixmap_.draw_line(drawGc_, long(di - pres), rcenter_ - (gint)pos,
 			       long(di), rcenter_ - pos1);
 	}
 
@@ -1011,45 +1066,46 @@ void SampleDisplay::updateSamples()
 	  pos = sampleBuf[len -1].left() * halfHeight;
 	  pos /= SHRT_MAX;
 	  if (pos != 0 || lastPosLeft != 0)
-	    pixmap_->draw_line(*drawGc_, long(di), lcenter_ - lastPosLeft, 
+	    pixmap_.draw_line(drawGc_, long(di), lcenter_ - lastPosLeft, 
 			       sampleEndX_, lcenter_ - (gint)pos);
 	  
 	  pos = sampleBuf[len - 1].right() * halfHeight;
 	  pos /= SHRT_MAX;
 	  if (pos != 0 || lastPosRight != 0)
-	    pixmap_->draw_line(*drawGc_, long(di), rcenter_ - lastPosRight,
+	    pixmap_.draw_line(drawGc_, long(di), rcenter_ - lastPosRight,
 			       sampleEndX_, rcenter_ - (gint)pos);
 	}
       }
 
       delete[] sampleBuf;
     }
+
   }
 
 
-  drawGc_->set_foreground(middleLineColor_);
+  drawGc_.set_foreground(middleLineColor_);
 
-  pixmap_->draw_line(*drawGc_, sampleStartX_, lcenter_,	sampleEndX_, lcenter_);
-  pixmap_->draw_line(*drawGc_, sampleStartX_, rcenter_, sampleEndX_, rcenter_);
+  pixmap_.draw_line(drawGc_, sampleStartX_, lcenter_,	sampleEndX_, lcenter_);
+  pixmap_.draw_line(drawGc_, sampleStartX_, rcenter_, sampleEndX_, rcenter_);
 
-  drawGc_->set_foreground(get_colormap().black());
+  drawGc_.set_foreground(get_colormap().black());
 
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, lcenter_ - halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, lcenter_ - halfHeight,
 		     sampleEndX_ + 1, lcenter_ - halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, lcenter_ + halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, lcenter_ + halfHeight,
 		     sampleEndX_ + 1, lcenter_ + halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, lcenter_ - halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, lcenter_ - halfHeight,
 		sampleStartX_ - 1, lcenter_ + halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleEndX_ + 1, lcenter_ - halfHeight,
+  pixmap_.draw_line(drawGc_, sampleEndX_ + 1, lcenter_ - halfHeight,
 		     sampleEndX_ + 1, lcenter_ + halfHeight);
 
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, rcenter_ - halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, rcenter_ - halfHeight,
 		     sampleEndX_ + 1, rcenter_ - halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, rcenter_ + halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, rcenter_ + halfHeight,
 		     sampleEndX_ + 1, rcenter_ + halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleStartX_ - 1, rcenter_ + halfHeight,
+  pixmap_.draw_line(drawGc_, sampleStartX_ - 1, rcenter_ + halfHeight,
 		     sampleStartX_ - 1, rcenter_ - halfHeight);
-  pixmap_->draw_line(*drawGc_, sampleEndX_ + 1, rcenter_ + halfHeight,
+  pixmap_.draw_line(drawGc_, sampleEndX_ + 1, rcenter_ + halfHeight,
 		     sampleEndX_ + 1, rcenter_ - halfHeight);
 
   drawTimeLine();
@@ -1063,7 +1119,7 @@ void SampleDisplay::updateSamples()
 
 void SampleDisplay::drawCursor(gint x)
 {
-  if (pixmap_ == NULL)
+  if (!pixmap_)
     return;
 
   if (x < sampleStartX_ || x > sampleEndX_)
@@ -1074,8 +1130,8 @@ void SampleDisplay::drawCursor(gint x)
   }
   
   if (!cursorDrawn_ || cursorX_ != x) {
-    drawGc_->set_foreground(cursorColor_);
-    get_window().draw_line(*drawGc_, x, trackLineY_, x, height_ - 1);
+    drawGc_.set_foreground(cursorColor_);
+    get_window().draw_line(drawGc_, x, trackLineY_, x, height_ - 1);
   }
   
   cursorDrawn_ = 1;
@@ -1097,7 +1153,7 @@ void SampleDisplay::drawTimeTick(gint x, gint y, unsigned long sample)
 {
   char buf[50];
 
-  if (pixmap_ == NULL)
+  if (!pixmap_)
     return;
 
   unsigned long min = sample / (60 * 44100);
@@ -1111,19 +1167,22 @@ void SampleDisplay::drawTimeTick(gint x, gint y, unsigned long sample)
 
   sprintf(buf, "%lu:%02lu:%02lu.%03lu", min, sec, frame, sample);
 
-  drawGc_->set_foreground(get_colormap().black());
+  drawGc_.set_foreground(get_colormap().black());
 
-  pixmap_->draw_line(*drawGc_, x, y - timeLineHeight_, x, y);
-  pixmap_->draw_string(*timeTickFont_, *drawGc_, x + 3, y - 3, buf);
+  pixmap_.draw_line(drawGc_, x, y - timeLineHeight_, x, y);
+  pixmap_.draw_string(timeTickFont_, drawGc_, x + 3, y - 3, buf);
 }
 
 gint SampleDisplay::timeTickWidth()
 {
-  return timeTickFont_->string_width("000:00:00.000") + 3;
+  return timeTickFont_.string_width("000:00:00.000") + 3;
 }
 
 void SampleDisplay::drawTimeLine()
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
   if (toc->length().lba() == 0)
@@ -1186,7 +1245,7 @@ void SampleDisplay::drawTimeLine()
 
 gint SampleDisplay::trackMarkerWidth()
 {
-  return trackMarkerFont_->string_width("00.00") + TRACK_MARKER_XPM_WIDTH + 2;
+  return trackMarkerFont_.string_width("00.00") + TRACK_MARKER_XPM_WIDTH + 2;
 }
 
 
@@ -1205,40 +1264,39 @@ void SampleDisplay::drawTrackMarker(int mode, gint x, int trackNr,
     Gdk_Pixmap *marker;
 
     if (extend) {
-      marker = indexNr == 1 ? trackExtendPixmap_ : indexExtendPixmap_;
+      marker = indexNr == 1 ? &trackExtendPixmap_ : &indexExtendPixmap_;
     }
     else {
       if (selected)
-	marker = indexNr == 1 ? trackMarkerSelectedPixmap_ :
-	                        indexMarkerSelectedPixmap_;
+	marker = indexNr == 1 ? &trackMarkerSelectedPixmap_ :
+	                        &indexMarkerSelectedPixmap_;
       else
-	marker = indexNr == 1 ? trackMarkerPixmap_ : indexMarkerPixmap_;
+	marker = indexNr == 1 ? &trackMarkerPixmap_ : &indexMarkerPixmap_;
     }
 
-
-
-    Gdk_Drawable dr(get_window());
+    Gdk_Window win(get_window());
+    Gdk_Drawable *dr = &win;
 
     if (mode == 0)
-      dr = *pixmap_;
+      dr = &pixmap_;
 
     if (mode == 0) {
       if (selected)
-	drawGc_->set_foreground(markerColor_);
+	drawGc_.set_foreground(markerColor_);
       else
-	drawGc_->set_foreground(get_colormap().white());
+	drawGc_.set_foreground(get_colormap().white());
 	
-      dr.draw_rectangle(*drawGc_, TRUE,  x-4, trackLineY_ - trackLineHeight_,
+      dr->draw_rectangle(drawGc_, TRUE,  x-4, trackLineY_ - trackLineHeight_,
 			trackMarkerWidth_, trackLineHeight_);
     }
 
-    drawGc_->set_foreground(get_colormap().black());
+    drawGc_.set_foreground(get_colormap().black());
 
-    dr.draw_pixmap(*drawGc_, *marker, 0, 0,
+    dr->draw_pixmap(drawGc_, *marker, 0, 0,
 		   x - 4, trackLineY_ - TRACK_MARKER_XPM_HEIGHT,
 		   TRACK_MARKER_XPM_WIDTH, TRACK_MARKER_XPM_HEIGHT);
 
-    dr.draw_string(*trackMarkerFont_, *drawGc_, 
+    dr->draw_string(trackMarkerFont_, drawGc_, 
 		    x + TRACK_MARKER_XPM_WIDTH / 2 + 2, trackLineY_ - 1, buf);
   }
   else {
@@ -1278,10 +1336,13 @@ void SampleDisplay::drawTrackLine()
 
 void SampleDisplay::updateTrackMarks()
 {
+  if (tocEdit_ == NULL)
+    return;
+
   Toc *toc = tocEdit_->toc();
 
-  drawGc_->set_foreground(get_colormap().white());
-  pixmap_->draw_rectangle(*drawGc_, TRUE,
+  drawGc_.set_foreground(get_colormap().white());
+  pixmap_.draw_rectangle(drawGc_, TRUE,
 			  sampleStartX_ - 4, trackLineY_ - trackLineHeight_,
 			  width_ - sampleStartX_, trackLineHeight_);
 
