@@ -1,6 +1,6 @@
 /*  cdrdao - write audio CD-Rs in disc-at-once mode
  *
- *  Copyright (C) 1998  Andreas Mueller <mueller@daneb.ping.de>
+ *  Copyright (C) 1998-2000  Andreas Mueller <mueller@daneb.ping.de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,15 +18,35 @@
  */
 /*
  * $Log: TocInfoDialog.cc,v $
+ * Revision 1.4  2000/04/23 09:07:08  andreasm
+ * * Fixed most problems marked with '//llanero'.
+ * * Added audio CD edit menus to MDIWindow.
+ * * Moved central storage of TocEdit object to MDIWindow.
+ * * AudioCdChild is now handled like an ordinary non modal dialog, i.e.
+ *   it has a normal 'update' member function now.
+ * * Added CdTextTable modal dialog.
+ * * Old functionality of xcdrdao is now available again.
+ *
+ * Revision 1.3  2000/04/16 20:31:20  andreasm
+ * Added missing stdio.h includes.
+ *
+ * Revision 1.2  2000/02/20 23:34:54  llanero
+ * fixed scsilib directory (files mising ?-()
+ * ported xdao to 1.1.8 / gnome (MDI) app
+ *
+ * Revision 1.1.1.1  2000/02/05 01:40:15  llanero
+ * Uploaded cdrdao 1.1.3 with pre10 patch applied.
+ *
  * Revision 1.1  1999/08/21 14:17:39  mueller
  * Initial revision
  *
  */
 
-static char rcsid[] = "$Id: TocInfoDialog.cc,v 1.1 1999/08/21 14:17:39 mueller Exp mueller $";
+static char rcsid[] = "$Id: TocInfoDialog.cc,v 1.4 2000/04/23 09:07:08 andreasm Exp $";
 
 #include "TocInfoDialog.h"
 
+#include <stdio.h>
 #include <stddef.h>
 #include <string.h>
 #include <ctype.h>
@@ -36,6 +56,7 @@ static char rcsid[] = "$Id: TocInfoDialog.cc,v 1.1 1999/08/21 14:17:39 mueller E
 #include "Toc.h"
 #include "CdTextItem.h"
 #include "TextEdit.h"
+#include "CdTextTable.h"
 
 #define MAX_CD_TEXT_LANGUAGE_CODES 22
 
@@ -111,21 +132,20 @@ static GenreCode CD_TEXT_GENRE_CODES[MAX_CD_TEXT_GENRE_CODES] = {
 TocInfoDialog::TocInfoDialog()
 {
   int i;
-  Gtk_Label *label;
-  Gtk_HBox *hbox;
-  Gtk_VBox *vbox, *vbox1;
-  Gtk_Frame *frame;
-  Gtk_Table *table;
-  Gtk_Button *button;
-  Gtk_VBox *contents = new Gtk_VBox;
-  Toc::TocType tocType;
+  Gtk::Label *label;
+  Gtk::HBox *hbox;
+  Gtk::VBox *vbox, *vbox1;
+  Gtk::Frame *frame;
+  Gtk::Table *table;
+  Gtk::Button *button;
+  Gtk::VBox *contents = new Gtk::VBox;
 
   tocEdit_ = NULL;
   active_ = 0;
   selectedTocType_ = Toc::CD_DA;
 
-  nofTracks_ = new Gtk_Label(string("99"));
-  tocLength_ = new Gtk_Label(string("100:00:00"));
+  nofTracks_ = new Gtk::Label(string("99"));
+  tocLength_ = new Gtk::Label(string("100:00:00"));
 
 
   catalog_ = new TextEdit("1234567890123");
@@ -135,61 +155,75 @@ TocInfoDialog::TocInfoDialog()
   catalog_->space(0);
   catalog_->digits(1);
 
-  tocTypeMenuFactory_ = new Gtk_ItemFactory_Menu("<Main>");
 
-  tocType = Toc::CD_DA;
-  tocTypeMenuFactory_->create_item("/CD-DA", 0, "<Item>", ItemFactoryConnector<TocInfoDialog, Toc::TocType>(this, &TocInfoDialog::setSelectedTocType, tocType));
+  Gtk::Menu *menu = manage(new Gtk::Menu);
+  Gtk::MenuItem *mi;
 
-  tocType = Toc::CD_ROM;
-  tocTypeMenuFactory_->create_item("/CD-ROM", 0, "<Item>", ItemFactoryConnector<TocInfoDialog, Toc::TocType>(this, &TocInfoDialog::setSelectedTocType, tocType));
+  mi = manage(new Gtk::MenuItem("CD-DA"));
+  mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedTocType),
+			    Toc::CD_DA));
+  mi->show();
+  menu->append(*mi);
 
-  tocType = Toc::CD_ROM_XA;
-  tocTypeMenuFactory_->create_item("/CD-ROM-XA", 0, "<Item>", ItemFactoryConnector<TocInfoDialog, Toc::TocType>(this, &TocInfoDialog::setSelectedTocType, tocType));
+  mi = manage(new Gtk::MenuItem("CD-ROM"));
+  mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedTocType),
+			    Toc::CD_ROM));
+  mi->show();
+  menu->append(*mi);
 
-  tocType = Toc::CD_I;
-  tocTypeMenuFactory_->create_item("/CD-I", 0, "<Item>", ItemFactoryConnector<TocInfoDialog, Toc::TocType>(this, &TocInfoDialog::setSelectedTocType, tocType));
+  mi = manage(new Gtk::MenuItem("CD-ROM-XA"));
+  mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedTocType),
+			    Toc::CD_ROM_XA));
+  mi->show();
+  menu->append(*mi);
 
-  tocType_ = new Gtk_OptionMenu;
-  tocType_->set_menu(tocTypeMenuFactory_->get_menu_widget(string("")));
+  mi = manage(new Gtk::MenuItem("CD-I"));
+  mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedTocType),
+			    Toc::CD_I));
+  mi->show();
+  menu->append(*mi);
+
+  tocType_ = new Gtk::OptionMenu;
+  tocType_->set_menu(menu);
 
   contents->set_spacing(10);
 
   // time data
-  frame = new Gtk_Frame(string("Summary"));
+  frame = new Gtk::Frame(string("Summary"));
 
-  table = new Gtk_Table(2, 2, FALSE);
+  table = new Gtk::Table(2, 2, FALSE);
   table->set_row_spacings(5);
   table->set_col_spacings(5);
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*table, FALSE, FALSE, 5);
-  vbox = new Gtk_VBox;
+  vbox = new Gtk::VBox;
   vbox->pack_start(*hbox, TRUE, TRUE, 5);
-  frame->add(vbox);
+  frame->add(*vbox);
   vbox->show();
   hbox->show();
   table->show();
   contents->pack_start(*frame, FALSE, FALSE);
   frame->show();
   
-  label = new Gtk_Label(string("Tracks:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Tracks:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE, FALSE);
   table->attach(*hbox, 0, 1, 0, 1);
   label->show();
   hbox->show();
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*nofTracks_, FALSE, FALSE);
   table->attach(*hbox, 1, 2, 0, 1);
   nofTracks_->show();
   hbox->show();
 
-  label = new Gtk_Label(string("Length:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Length:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE, FALSE);
   table->attach(*hbox, 0, 1, 1, 2);
   label->show();
   hbox->show();
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*tocLength_, FALSE, FALSE);
   table->attach(*hbox, 1, 2, 1, 2);
   tocLength_->show();
@@ -198,13 +232,13 @@ TocInfoDialog::TocInfoDialog()
 
 
   // sub-channel data
-  frame = new Gtk_Frame(string("Sub-Channel"));
+  frame = new Gtk::Frame(string("Sub-Channel"));
 
-  vbox = new Gtk_VBox;
+  vbox = new Gtk::VBox;
   vbox->set_spacing(0);
 
-  hbox = new Gtk_HBox;
-  label = new Gtk_Label(string("Toc Type: "));
+  hbox = new Gtk::HBox;
+  label = new Gtk::Label(string("Toc Type: "));
   hbox->pack_start(*label, FALSE);
   label->show();
   hbox->pack_start(*tocType_, FALSE);
@@ -212,8 +246,8 @@ TocInfoDialog::TocInfoDialog()
   vbox->pack_start(*hbox);
   hbox->show();
 
-  hbox = new Gtk_HBox;
-  label = new Gtk_Label(string("UPC/EAN: "));
+  hbox = new Gtk::HBox;
+  label = new Gtk::Label(string("UPC/EAN: "));
   hbox->pack_start(*label, FALSE);
   label->show();
   hbox->pack_start(*catalog_, FALSE);
@@ -221,11 +255,11 @@ TocInfoDialog::TocInfoDialog()
   vbox->pack_start(*hbox);
   hbox->show();
   
-  vbox1 = new Gtk_VBox;
+  vbox1 = new Gtk::VBox;
   vbox1->pack_start(*vbox, TRUE, TRUE, 5);
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*vbox1, TRUE, TRUE, 5);
-  frame->add(hbox);
+  frame->add(*hbox);
   vbox->show();
   vbox1->show();
   hbox->show();
@@ -234,20 +268,20 @@ TocInfoDialog::TocInfoDialog()
   frame->show();
 
   // CD-TEXT data
-  frame = new Gtk_Frame(string("CD-TEXT"));
+  frame = new Gtk::Frame(string("CD-TEXT"));
 
-  Gtk_Notebook *notebook = new Gtk_Notebook;
+  Gtk::Notebook *notebook = new Gtk::Notebook;
 
   for (i = 0; i < 8; i++) {
     vbox = createCdTextPage(i);
-    notebook->append_page(*vbox, *(cdTextPages_[i].label));
+    notebook->pages().push_back(Gtk::Notebook_Helpers::TabElem(*vbox, *(cdTextPages_[i].label)));
   }
 
-  vbox1 = new Gtk_VBox;
+  vbox1 = new Gtk::VBox;
   vbox1->pack_start(*notebook, FALSE, FALSE, 5);
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*vbox1, TRUE, TRUE, 5);
-  frame->add(hbox);
+  frame->add(*hbox);
   notebook->show();
   vbox1->show();
   hbox->show();
@@ -257,7 +291,7 @@ TocInfoDialog::TocInfoDialog()
 
 
 
-  hbox = new Gtk_HBox;
+  hbox = new Gtk::HBox;
   hbox->pack_start(*contents, TRUE, TRUE, 10);
   contents->show();
   get_vbox()->pack_start(*hbox, TRUE, TRUE, 10);
@@ -265,17 +299,17 @@ TocInfoDialog::TocInfoDialog()
 
   get_vbox()->show();
 
-  Gtk_HButtonBox *bbox = new Gtk_HButtonBox(GTK_BUTTONBOX_SPREAD);
+  Gtk::HButtonBox *bbox = new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD);
 
-  applyButton_ = new Gtk_Button(string(" Apply "));
+  applyButton_ = new Gtk::Button(string(" Apply "));
   bbox->pack_start(*applyButton_);
   applyButton_->show();
-  connect_to_method(applyButton_->clicked, this, &TocInfoDialog::applyAction);
+  applyButton_->clicked.connect(SigC::slot(this,&TocInfoDialog::applyAction));
 
-  button = new Gtk_Button(string(" Cancel "));
+  button = new Gtk::Button(string(" Cancel "));
   bbox->pack_start(*button);
   button->show();
-  connect_to_method(button->clicked, this, &TocInfoDialog::cancelAction);
+  button->clicked.connect(SigC::slot(this,&TocInfoDialog::cancelAction));
 
   get_action_area()->pack_start(*bbox);
   bbox->show();
@@ -376,18 +410,21 @@ void TocInfoDialog::createCdTextLanguageMenu(int n)
   int i;
 
   bval.block = n;
-  cdTextPages_[n].languageMenuFactory = new Gtk_ItemFactory_Menu("<Main>");
+
+  Gtk::Menu *menu = manage(new Gtk::Menu);
+  Gtk::MenuItem *mi;
 
   for (i = 0; i < MAX_CD_TEXT_LANGUAGE_CODES; i++) {
-    string s("/");
-    s += CD_TEXT_LANGUAGE_CODES[i].name;
     bval.value = i;
-    cdTextPages_[n].languageMenuFactory->create_item(s, 0, "<Item>", ItemFactoryConnector<TocInfoDialog, TocInfoDialog::BlockValue>(this, &TocInfoDialog::setSelectedCDTextLanguage, bval));
+
+    mi = manage(new Gtk::MenuItem(CD_TEXT_LANGUAGE_CODES[i].name));
+    mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedCDTextLanguage), bval));
+    mi->show();
+    menu->append(*mi);
   }
 
-  cdTextPages_[n].language = new Gtk_OptionMenu;
-  cdTextPages_[n].language->set_menu(cdTextPages_[n].languageMenuFactory->get_menu_widget(string("")));
-
+  cdTextPages_[n].language = new Gtk::OptionMenu;
+  cdTextPages_[n].language->set_menu(menu);
 }
 
 void TocInfoDialog::createCdTextGenreMenu(int n)
@@ -396,42 +433,47 @@ void TocInfoDialog::createCdTextGenreMenu(int n)
   int i;
 
   bval.block = n;
-  cdTextPages_[n].genreMenuFactory = new Gtk_ItemFactory_Menu("<Main>");
+
+  Gtk::Menu *menu = manage(new Gtk::Menu);
+  Gtk::MenuItem *mi;
 
   for (i = 0; i < MAX_CD_TEXT_GENRE_CODES; i++) {
-    string s("/");
-    s += CD_TEXT_GENRE_CODES[i].name;
     bval.value = i;
-    cdTextPages_[n].genreMenuFactory->create_item(s, 0, "<Item>", ItemFactoryConnector<TocInfoDialog, TocInfoDialog::BlockValue>(this, &TocInfoDialog::setSelectedCDTextGenre, bval));
+
+    mi = manage(new Gtk::MenuItem(CD_TEXT_GENRE_CODES[i].name));
+    mi->activate.connect(bind(slot(this, &TocInfoDialog::setSelectedCDTextGenre), bval));
+    mi->show();
+    menu->append(*mi);
   }
 
-  cdTextPages_[n].genre = new Gtk_OptionMenu;
-  cdTextPages_[n].genre->set_menu(cdTextPages_[n].genreMenuFactory->get_menu_widget(string("")));
-
+  cdTextPages_[n].genre = new Gtk::OptionMenu;
+  cdTextPages_[n].genre->set_menu(menu);
 }
 
 
-Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
+Gtk::VBox *TocInfoDialog::createCdTextPage(int n)
 {
   char buf[20];
-  Gtk_Table *table = new Gtk_Table(11, 2, FALSE);
-  Gtk_VBox *vbox = new Gtk_VBox;
-  Gtk_HBox *hbox;
-  Gtk_Label *label;
+  Gtk::Table *table = new Gtk::Table(11, 2, FALSE);
+  Gtk::VBox *vbox = new Gtk::VBox;
+  Gtk::VBox *vbox1;
+  Gtk::HBox *hbox;
+  Gtk::Label *label;
+  Gtk::Button *button;
 
   sprintf(buf, "%d", n);
-  cdTextPages_[n].label = new Gtk_Label(string(buf));
+  cdTextPages_[n].label = new Gtk::Label(string(buf));
   cdTextPages_[n].label->show();
 
-  cdTextPages_[n].title = new Gtk_Entry;
-  cdTextPages_[n].performer = new Gtk_Entry;
-  cdTextPages_[n].songwriter = new Gtk_Entry;
-  cdTextPages_[n].composer = new Gtk_Entry;
-  cdTextPages_[n].arranger = new Gtk_Entry;
-  cdTextPages_[n].message = new Gtk_Entry;
-  cdTextPages_[n].catalog = new Gtk_Entry;
-  cdTextPages_[n].upcEan = new Gtk_Entry;
-  cdTextPages_[n].genreInfo = new Gtk_Entry;
+  cdTextPages_[n].title = new Gtk::Entry;
+  cdTextPages_[n].performer = new Gtk::Entry;
+  cdTextPages_[n].songwriter = new Gtk::Entry;
+  cdTextPages_[n].composer = new Gtk::Entry;
+  cdTextPages_[n].arranger = new Gtk::Entry;
+  cdTextPages_[n].message = new Gtk::Entry;
+  cdTextPages_[n].catalog = new Gtk::Entry;
+  cdTextPages_[n].upcEan = new Gtk::Entry;
+  cdTextPages_[n].genreInfo = new Gtk::Entry;
 
   createCdTextLanguageMenu(n);
   createCdTextGenreMenu(n);
@@ -440,8 +482,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->set_col_spacings(5);
   table->show();
 
-  label = new Gtk_Label(string("Language:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Language:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 0, 1, GTK_FILL);
   hbox->show();
@@ -449,8 +491,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].language), 1, 2, 0, 1);
   cdTextPages_[n].language->show();
 
-  label = new Gtk_Label(string("Title:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Title:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 1, 2, GTK_FILL);
   hbox->show();
@@ -458,8 +500,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].title), 1, 2, 1, 2);
   cdTextPages_[n].title->show();
 
-  label = new Gtk_Label(string("Performer:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Performer:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 2, 3, GTK_FILL);
   hbox->show();
@@ -467,8 +509,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].performer), 1, 2, 2, 3);
   cdTextPages_[n].performer->show();
 
-  label = new Gtk_Label(string("Songwriter:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Songwriter:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 3, 4, GTK_FILL);
   hbox->show();
@@ -476,8 +518,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].songwriter), 1, 2, 3, 4);
   cdTextPages_[n].songwriter->show();
 
-  label = new Gtk_Label(string("Composer:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Composer:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 4, 5, GTK_FILL);
   hbox->show();
@@ -485,8 +527,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].composer), 1, 2, 4, 5);
   cdTextPages_[n].composer->show();
 
-  label = new Gtk_Label(string("Arranger:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Arranger:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 5, 6, GTK_FILL);
   hbox->show();
@@ -494,8 +536,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].arranger), 1, 2, 5, 6);
   cdTextPages_[n].arranger->show();
 
-  label = new Gtk_Label(string("Message:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Message:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 6, 7, GTK_FILL);
   hbox->show();
@@ -503,8 +545,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].message), 1, 2, 6, 7);
   cdTextPages_[n].message->show();
 
-  label = new Gtk_Label(string("Catalog:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Catalog:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 7, 8, GTK_FILL);
   hbox->show();
@@ -512,8 +554,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].catalog), 1, 2, 7, 8);
   cdTextPages_[n].catalog->show();
 
-  label = new Gtk_Label(string("UPC/EAN:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("UPC/EAN:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 8, 9, GTK_FILL);
   hbox->show();
@@ -521,8 +563,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].upcEan), 1, 2, 8, 9);
   cdTextPages_[n].upcEan->show();
 
-  label = new Gtk_Label(string("Genre:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Genre:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 9, 10, GTK_FILL);
   hbox->show();
@@ -530,8 +572,8 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   table->attach(*(cdTextPages_[n].genre), 1, 2, 9, 10);
   cdTextPages_[n].genre->show();
 
-  label = new Gtk_Label(string("Genre Info:"));
-  hbox = new Gtk_HBox;
+  label = new Gtk::Label(string("Genre Info:"));
+  hbox = new Gtk::HBox;
   hbox->pack_end(*label, FALSE);
   table->attach(*hbox, 0, 1, 10, 11, GTK_FILL);
   hbox->show();
@@ -540,9 +582,39 @@ Gtk_VBox *TocInfoDialog::createCdTextPage(int n)
   cdTextPages_[n].genreInfo->show();
 
   vbox->pack_start(*table, FALSE);
+
+  button = new Gtk::Button(" Entry by Table ");
+  Gtk::HButtonBox *bbox = new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD);
+  bbox->pack_start(*button);
+  button->show();
+  button->clicked.connect(bind(slot(this, &TocInfoDialog::cdTextTableAction),
+			       n));
+  vbox->pack_start(*bbox, TRUE);
+  bbox->show();
+
+
+  hbox = new Gtk::HBox;
+  hbox->pack_start(*vbox, TRUE, TRUE, 3);
   vbox->show();
 
-  return vbox;
+  vbox1 = new Gtk::VBox;
+  vbox1->pack_start(*hbox, TRUE, TRUE, 3);
+  hbox->show();
+
+  vbox1->show();
+  
+  return vbox1;
+}
+
+void TocInfoDialog::cdTextTableAction(int language)
+{
+  if (tocEdit_ != NULL && tocEdit_->editable()) {
+    CdTextTable table(tocEdit_, language);
+
+    if (table.run()) {
+      guiUpdate();
+    }
+  }
 }
 
 gint TocInfoDialog::delete_event_impl(GdkEventAny*)
